import collections
import ipaddress

import ipsec
import openvpn
import openvpn_mikrotik
import wireguard
from Marks import Marks
from Packet import *


class MainAnalyzer:

    def __init__(self, filters, sniffer):
        self.sniffer = sniffer
        self.ip_addresses = collections.defaultdict(
            lambda: collections.defaultdict(lambda: collections.defaultdict(int)))
        self.analyzers = []
        if Marks.IPSEC in filters:
            self.analyzers.append(ipsec.IPSecAnalyzer())
        if Marks.OPENVPN in filters:
            self.analyzers.append(openvpn.OpenVPNAnalyzer())
        if Marks.WIREGUARD in filters:
            self.analyzers.append(wireguard.WireguardAnalyzer())
        if Marks.OPENVPN_MIKROTIK in filters:
            self.analyzers.append(openvpn_mikrotik.OpenVPNMikrotikAnalyzer())

    # pridavani poctu vpn packetu do struktury pro vypis
    def add_vpn_packet_to_dict(self, packet, network, count=1):
        if packet is None:
            return -2

        src_ip = get_src_ip_from_packet(packet)
        dst_ip = get_dst_ip_from_packet(packet)
        src_port = get_src_port_from_packet(packet)
        dst_port = get_dst_port_from_packet(packet)
        if (get_mark_from_packet(packet) == Marks.IPSEC_CONTROL) or (get_mark_from_packet(packet) == Marks.IPSEC_DATA):
            src_ip_str = str(src_ip)
            dst_ip_str = str(dst_ip)
        else:
            src_ip_str = f"{str(src_ip)}:{str(src_port)}"
            dst_ip_str = f"{str(dst_ip)}:{str(dst_port)}"
        if ipaddress.IPv4Address(src_ip) in ipaddress.IPv4Network(network) or ipaddress.IPv4Address(
                dst_ip) in ipaddress.IPv4Network(network):
            if src_ip_str > dst_ip_str:
                ip_port = dst_ip_str + ";" + src_ip_str
                direction = "PACKETS_B"
            else:
                ip_port = src_ip_str + ";" + dst_ip_str
                direction = "PACKETS_A"
            if not ip_port in self.ip_addresses[get_mark_from_packet(packet)]:
                self.ip_addresses[get_mark_from_packet(packet)][ip_port][direction] = count
            else:
                self.ip_addresses[get_mark_from_packet(packet)][ip_port][direction] += count

    # analyza packetu a hledani vpn
    def start_analysis(self, network, packet_queue):

        while True:
            packet = packet_queue.get()
            packet = self.sniffer.filter_packets(packet, network)
            if packet is None:
                packet_queue.task_done()
                continue
            for analyzer in self.analyzers:
                count = analyzer.analyze(packet)
                if count > 0:
                    self.add_vpn_packet_to_dict(packet, network, count)
            packet_queue.task_done()

    def post_processing(self):
        for analyzer in self.analyzers:
            analyzer.filter_false_positives(self.ip_addresses)
